----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    21:32:34 05/25/2016 
-- Design Name: 
-- Module Name:    ADC_channel_selector - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

-----------------------------------------------------------------------------------------------------------------
--this state machine periodically selects channels of ADC
-----------------------------------------------------------------------------------------------------------------

entity ADC_channel_selector is
    Port ( clk 					: in  STD_LOGIC;
           rst 					: in  STD_LOGIC;
			  spi_busy 				: in std_logic;								--spi cannot receive new data
			  data_in 				: in std_logic_vector(11 downto 0);
			  new_data_available : out std_logic;								--tells the SPI interface it can start sending data
           channel_number 		: out  STD_LOGIC_VECTOR (3 downto 0);	--channel no. X which has to be selected
           vin 					: out  STD_LOGIC_VECTOR (7 downto 0);  --inpu voltage for power calculation
           iin 					: out  STD_LOGIC_VECTOR (7 downto 0);  --input current for the same reason
           output_sine 			: out  STD_LOGIC_VECTOR (7 downto 0);  --sensed value of the output sinusoid, not used any further
           mains_sine 			: out  STD_LOGIC_VECTOR (7 downto 0)); --sensed value of mains voltage, not use any further
end ADC_channel_selector;

architecture Behavioral of ADC_channel_selector is

type state_t is (idle, ch0,ch1,ch2, ch3);
signal state : state_t;

begin

	process(clk,rst) begin
	  if (rst = '1') then
	    state <= idle;
		 new_data_available <= '0';
		 vin <= (others => '0');
		 iin <= (others => '0');
		 output_sine <= (others => '0');
		 mains_sine <= (others => '0');
		 channel_number <= (others => '0');
     elsif rising_edge(clk) then
	    case state is
			when idle => 
				new_data_available <= '0';
				state <= ch0;
			when ch0 =>
			   --new_data_available <= '1';
				channel_number     <= "0000";
				if spi_busy = '1' then 
				  new_data_available <= '0';
				else
              new_data_available <= '1';
				  state <= ch1;
				  vin <= data_in(7 downto 0);
            end if;
			when ch1 =>
            channel_number    <= "0001";
            if spi_busy = '1'	then
              new_data_available <= '0';
				else 
              new_data_available <= '1';
				  state <= ch2;
				  iin <= data_in(7 downto 0);
            end if;
				
			when ch2 =>
            channel_number    <= "0010";
            if spi_busy = '1'	then
              new_data_available <= '0';
				else 
              new_data_available <= '1';
				  state <= ch3;
				  output_sine <= data_in(7 downto 0);
            end if;
			when ch3 =>
            channel_number    <= "0011";
            if spi_busy = '1'	then
              new_data_available <= '0';
				else 
              new_data_available <= '1';
				  state <= ch0;
				  mains_sine <=  data_in(7 downto 0);
            end if;				
			when others =>
				state <= idle;
			end case;
		 end if;
		end process;	
end Behavioral;

