----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    23:43:00 03/15/2016 
-- Design Name: 
-- Module Name:    MPPT_block - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;
--use std_logic_unsigned.all;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

---------------------------------------------------------------------------------------
--this module is based on current sweep MPPT algorithm
--the duty cycle is increased until the power from the solar panel ceases to increase
----------------------------------------------------------------------------------------

entity MPPT_block is
    Port ( 	clk 							: 			in std_logic;
				rst 							: 			in std_logic;
				new_adc_data 				:		   in std_logic;										-----tells if there is new data to be processed
				vin 							: 			in  STD_LOGIC_VECTOR (7 downto 0);			-----input voltage
				iin 							: 			in  STD_LOGIC_VECTOR (7 downto 0);			-----input current
				d_out 						: 			out  STD_LOGIC_VECTOR (9 downto 0));		-----resulting duty cycle
end MPPT_block;

---------------------------------------------------------------------------------------------------------------------------------------
--signal and const declaration
---------------------------------------------------------------------------------------------------------------------------------------

architecture Behavioral of MPPT_block is
constant RESET_CNT_WIDTH			   : natural := 28;
constant RESET_VALUE 					: natural := 250000; 
constant PWM_WIDTH 						: natural := 10;
signal pin_delayed 						: std_logic_vector(15 downto 0);	----delayed sample of calculated input power
signal timer_cnt 							: std_logic_vector(RESET_CNT_WIDTH-1 downto 0);
signal d_out_signal 						: std_logic_vector(PWM_WIDTH-1 downto 0) := (others => '0');
signal pin 									: std_logic_vector(15 downto 0);							----multiplied input current and voltage

type state_t is (init, searching_mppt, MPPT);
signal state : state_t;
begin

d_out <= d_out_signal;
--------------------memory block
process(clk,rst)
begin
	if rst = '1' then
		pin_delayed <= (others => '0');
		d_out_signal <= (others => '0');
		state <= init;
		timer_cnt <= (others => '0');
	elsif rising_edge(clk) then
		if new_adc_data = '1' then
			pin_delayed <= pin;
		end if;
---------------------------------------
---------------------------------------
--state machine case block
---------------------------------------
---------------------------------------		
		case state is
		when init =>
			d_out_signal <= (others => '0');
			state <= searching_mppt; 
		when searching_mppt =>
			if new_adc_data = '1' then
				if pin_delayed > pin then --------------------current supplied from the panel decreased
					state <= MPPT;
				else 
					d_out_signal <= std_logic_vector(unsigned(d_out_signal) + 1); ---------increase duty cycle as long as the MPP is not reached
				end if;
			end if;
		when MPPT =>
			if (timer_cnt < std_logic_vector(to_unsigned(RESET_VALUE,RESET_CNT_WIDTH))) then ---turn on reset counter, which initializes MPPT search update
				timer_cnt <= std_logic_vector(unsigned(timer_cnt) + 1);
			else
				timer_cnt <= (others => '0');
				state <= init;
			end if;
		when others =>
			state <= init;
	  end case;		
	end if;
end process;
-------------------end of memory block

-------------------input current and voltage multiplication
process(vin,iin)
begin
	pin <= std_logic_vector((unsigned(vin)*unsigned(iin)));
end process;	
-------------------end of input current and voltage multiplication



end Behavioral;

