----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    20:08:56 03/13/2016 
-- Design Name: 
-- Module Name:    boost_pwm_counter - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments:  
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity boost_pwm_counter is
    Port ( clk,rst : in  STD_LOGIC;
           duty_cyc_in : in  STD_LOGIC_VECTOR (9 downto 0);
           shutdown : in  STD_LOGIC;
           gate_pulse_out : out  STD_LOGIC_vector(3 downto 0));
end boost_pwm_counter;

architecture Behavioral of boost_pwm_counter is
signal q_cnt_1,d_cnt_1,q_stage_delay : std_logic_vector(9 downto 0);
signal q_cnt_2,d_cnt_2,d_stage_delay : std_logic_vector(9 downto 0);
signal q_cnt_3,d_cnt_3 : std_logic_vector(9 downto 0);
signal q_cnt_4,d_cnt_4 : std_logic_vector(9 downto 0);
signal q_cnt1_triggered,q_cnt2_triggered,q_cnt3_triggered,q_cnt4_triggered : std_logic;
signal d_cnt1_triggered,d_cnt2_triggered,d_cnt3_triggered,d_cnt4_triggered : std_logic;
signal trigger : std_logic_vector(3 downto 0);
signal q_shutdown,d_shutdown : std_logic;
signal q_shutdown_cnt,d_shutdown_cnt : std_logic_vector(2 downto 0);
begin
	process(clk,rst)
	begin
		if rst = '1' then
			q_cnt_1 <= (others => '0');
			q_cnt_2 <= (others => '0');
			q_cnt_3 <= (others => '0');
			q_cnt_4 <= (others => '0');
			q_cnt1_triggered <= '0';
			q_cnt2_triggered <= '0';
			q_cnt3_triggered <= '0';
			q_cnt4_triggered <= '0';
			q_stage_delay <= (others => '0');
			q_shutdown <= '0';
			q_shutdown_cnt <= (others => '0');  
		elsif rising_edge(clk) then
			q_cnt_1 <= d_cnt_1; 
			q_cnt_2 <= d_cnt_2;
			q_cnt_3 <= d_cnt_3;
			q_cnt_4 <= d_cnt_4;
			q_cnt1_triggered <= d_cnt1_triggered;
			q_cnt2_triggered <= d_cnt2_triggered;
			q_cnt3_triggered <= d_cnt3_triggered;
			q_cnt4_triggered <= d_cnt4_triggered;
			q_stage_delay <= d_stage_delay;
			q_shutdown <= d_shutdown;
			q_shutdown_cnt <= d_shutdown_cnt;  
		end if;
	end process;
	
	process(q_cnt_1,q_cnt1_triggered)
	begin
		d_cnt_1<=q_cnt_1;
		if q_cnt1_triggered = '1' then
			d_cnt_1 <= std_logic_vector(unsigned(q_cnt_1) + 1);
		end if;
	end process;

	process(q_cnt_2,q_cnt2_triggered)
	begin
		d_cnt_2<=q_cnt_2;
		if q_cnt2_triggered = '1' then
			d_cnt_2 <= std_logic_vector(unsigned(q_cnt_2) + 1);
		end if;
	end process;

	process(q_cnt_3,q_cnt3_triggered)
	begin
		d_cnt_3<=q_cnt_3;
		if q_cnt3_triggered = '1' then
			d_cnt_3 <= std_logic_vector(unsigned(q_cnt_3) + 1);
		end if;
	end process;	

	process(q_cnt_4,q_cnt4_triggered)
	begin
		d_cnt_4<=q_cnt_4;
		if q_cnt4_triggered = '1' then
			d_cnt_4 <= std_logic_vector(unsigned(q_cnt_4) + 1);
		end if;
	end process;

	process(q_cnt_1,q_cnt_2,q_cnt_3,q_cnt_4,duty_cyc_in,q_cnt1_triggered,q_cnt2_triggered,q_cnt3_triggered,q_cnt4_triggered,q_shutdown)
	begin
		gate_pulse_out <= (others => '1');
		if (q_cnt_1 > duty_cyc_in) and q_cnt1_triggered = '1' and q_shutdown = '0' then 
			gate_pulse_out(0) <= '0';
		end if;
		if (q_cnt_2 > duty_cyc_in) and q_cnt2_triggered = '1' and q_shutdown = '0' then 
			gate_pulse_out(1) <= '0';
		end if;
		if (q_cnt_3 > duty_cyc_in) and q_cnt3_triggered = '1' and q_shutdown = '0' then 
			gate_pulse_out(2) <= '0';
		end if;
		if (q_cnt_4 > duty_cyc_in) and q_cnt4_triggered = '1' and q_shutdown = '0' then
			gate_pulse_out(3) <= '0';
		end if;
	end process;

	process(trigger,q_cnt1_triggered,q_cnt2_triggered,q_cnt3_triggered,q_cnt4_triggered)
	begin
		d_cnt1_triggered <= q_cnt1_triggered;
		d_cnt2_triggered <= q_cnt2_triggered;
		d_cnt3_triggered <= q_cnt3_triggered;
 		d_cnt4_triggered <= q_cnt4_triggered;
		if trigger = X"1" then
			d_cnt1_triggered <= '1';
		elsif trigger = X"2" then
			d_cnt2_triggered <= '1';
		elsif trigger = X"4" then
			d_cnt3_triggered <= '1';
		elsif trigger = X"8" then
			d_cnt4_triggered <= '1';
		end if;
	end process;

	process(q_stage_delay)
	begin
		d_stage_delay<=q_stage_delay;
		if q_stage_delay < std_logic_vector(to_unsigned(1022,10)) then
			d_stage_delay <= std_logic_vector(unsigned(q_stage_delay) + 1);
		end if;
	end process;
	
	process(q_stage_delay)
	begin
		trigger <= (others => '0');
		if q_stage_delay = std_logic_vector(to_unsigned(0,10)) then
			trigger <= X"1";
		elsif q_stage_delay = std_logic_vector(to_unsigned(255,10)) then
			trigger <= X"2";
		elsif q_stage_delay = std_logic_vector(to_unsigned(511,10)) then
			trigger <= X"4";
		elsif q_stage_delay = std_logic_vector(to_unsigned(767,10)) then
			trigger <= X"8";
		end if;
	end process;

	process(q_shutdown_cnt,shutdown)
	begin
		d_shutdown_cnt <= q_shutdown_cnt;
		if shutdown = '1' then
			if q_shutdown_cnt < std_logic_vector(to_unsigned(5,3)) then
				d_shutdown_cnt <= std_logic_vector(unsigned(q_shutdown_cnt) + 1);
			end if;
		end if;
	end process;
	
	process(q_shutdown_cnt,q_shutdown)
	begin
		d_shutdown <= q_shutdown; 
		if	q_shutdown_cnt = std_logic_vector(to_unsigned(5,3)) then													-------------100ns shutdown delay
			d_shutdown <= '1';
		end if;
	end process;	
	
	--shutdown <= q_shutdown;
		
	
end Behavioral;

