----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    19:14:50 03/20/2016 
-- Design Name: 
-- Module Name:    sine_pwm_counter - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity sine_pwm_counter is
    Port ( clk 										: in  STD_LOGIC;
           rst 										: in  STD_LOGIC;
           sine_sample_in,phase_in 				: in  STD_LOGIC_VECTOR (9 downto 0);
           pwm_out_1 								: out  STD_LOGIC;
           pwm_out_2,pwm_out_3,pwm_out_4 		: out  STD_LOGIC);
end sine_pwm_counter;

architecture Behavioral of sine_pwm_counter is
signal q_cnt 											: std_logic_vector(9 downto 0);
constant minimum_dead_time 						: natural := 0;
constant initial_dead_time 						: natural := 50;		--overflow will occur if greater than 500, which leads to almost 100 percent duty cycle of the first converter diagonal
signal q_divider,d_divider 						: std_logic_vector(15 downto 0);
signal q_dtcounter,d_dtcounter 					: std_logic_vector(9 downto 0);
signal CE : std_logic;
begin

	process(clk, rst)
	begin
		if rst = '1' then 
			q_divider <= (others => '0');
			q_dtcounter <= std_logic_vector(to_unsigned(initial_dead_time,10));--(others => '0');
		elsif rising_edge(clk) then
			q_divider <= d_divider; 
			q_dtcounter <= d_dtcounter;
		end if;
	end process;
	
	process(q_divider)
	begin
		CE <= '0';
		d_divider <= q_divider;
		if q_divider < X"FFFF" then
			d_divider <= std_logic_vector(unsigned(q_divider)+1);
		elsif q_divider = X"FFFF" then
			CE <= '1';
			d_divider <= (others =>'0'); 
		end if;
	end process;

		process(CE,q_dtcounter)
		begin

		d_dtcounter <= q_dtcounter;
			if CE = '1' then
				if q_dtcounter > std_logic_vector(to_unsigned(0,10)) then
					d_dtcounter <= std_logic_vector(unsigned(q_dtcounter)-1);
				end if;	
			end if;		
		end process;	
	q_cnt <= phase_in;-- 
	
	process(q_cnt,sine_sample_in,q_dtcounter)
	begin
		pwm_out_1 <= '0';
		pwm_out_2 <= '0';
		pwm_out_3 <= '1';
		pwm_out_4 <= '1';
		if (q_cnt < sine_sample_in) then
			if q_cnt > std_logic_vector(to_unsigned(minimum_dead_time,10)+unsigned(q_dtcounter)) then
				pwm_out_1 <= '1';
				pwm_out_3 <= '0';
			end if;	
			--pwm_out_2 <= '0';
		end if;
		if q_cnt > std_logic_vector(unsigned(sine_sample_in) + to_unsigned(minimum_dead_time,10) + unsigned(q_dtcounter)) then
			--if q_cnt < std_logic_vector(to_unsigned(1000,10)) then
				pwm_out_2 <= '1';
				pwm_out_4 <= '0';
			--end if;	
		end if;
	end process;

end Behavioral;

