----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    18:15:42 03/20/2016 
-- Design Name: 
-- Module Name:    sine_sample_limiter - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

--------------------------------------------------------------------------------------------------------------
--This module decreases the peak value of sine wave, in order to avoid output bridge crossconductions
--------------------------------------------------------------------------------------------------------------
entity sine_sample_limiter is
    Port ( sine_in 				: in  STD_LOGIC_VECTOR (9 downto 0);
				clk,rst 				: in std_logic;
           sine_out 				: out  STD_LOGIC_VECTOR (9 downto 0);
			  sine_amplitude 		: in std_logic_vector (9 downto 0));
end sine_sample_limiter;

architecture Behavioral of sine_sample_limiter is
signal sine_out_1 				: std_logic_vector(19 downto 0);
signal sine_out_2 				: std_logic_vector(9 downto 0);
signal offset_compensation 	: std_logic_vector(19 downto 0);
constant sine_offset 			: natural := 512; 							--FS/2
constant sine_offset_shifted 	: natural := 524288;							--FS/2 * 1024
begin

	process(sine_amplitude)
	begin
		offset_compensation <= std_logic_vector(unsigned(std_logic_vector(to_unsigned(sine_offset_shifted,20)))-unsigned(std_logic_vector(to_unsigned(sine_offset,10)*unsigned(sine_amplitude))));
	end process;
	
	process(offset_compensation,sine_out_1)
	begin
		sine_out_2 <= std_logic_vector(unsigned(sine_out_1(19 downto 10)) + unsigned(offset_compensation(19 downto 10)));
	end process;	

	process(sine_in,sine_amplitude)
	begin
		sine_out_1 <= std_logic_vector(unsigned(sine_in(9 downto 0)) * unsigned(sine_amplitude));
	end process;
	
	process(clk,rst)
	begin
		if rst = '1' then
			sine_out <= (others => '0');
		elsif	rising_edge(clk) then
			sine_out <= sine_out_2(9 downto 0);
		end if;
	end process;
		

end Behavioral;

