----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    14:54:46 03/17/2016 
-- Design Name: 
-- Module Name:    softstart_circuit - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

---------------------------------------------------------------------------------------------------------------
--this module decreases actual duty cycle of transistor in boost board stage at the start up to prevent surges
---------------------------------------------------------------------------------------------------------------
entity softstart_circuit is
    Port ( clk 				: in  STD_LOGIC;
           rst 				: in  STD_LOGIC;
			  duty_cycle 		: in std_logic_vector(9 downto 0);
           softstart_out 	: out  STD_LOGIC_VECTOR (9 downto 0));
end softstart_circuit;

architecture Behavioral of softstart_circuit is
constant delay_const 										: integer := 1000;
constant output_final_value 								: integer := 100;
signal q_softstart_out,d_softstart_out 				: std_logic_vector(9 downto 0);
signal d_softstart_stage,q_softstart_stage 			: std_logic;
signal d_softstart_counter,q_softstart_counter 		: std_logic_vector(9 downto 0);
signal clock_enable : std_logic;
begin

--softstart_out <= q_softstart_out;

	process(clk,rst)
	begin
		if rst = '1' then
			q_softstart_out <= (others => '1');
			q_softstart_counter <= (others => '0');
			q_softstart_stage <= '0';
		elsif rising_edge(clk) then
			q_softstart_out <= d_softstart_out;
			q_softstart_stage <= d_softstart_stage;
			q_softstart_counter <= d_softstart_counter;
		end if;
	end process;
	
	process(q_softstart_stage,q_softstart_counter)
	begin
		d_softstart_counter <= q_softstart_counter;
		clock_enable <= '0';
		if q_softstart_counter < std_logic_vector(to_unsigned(delay_const,10)) then
			d_softstart_counter <= std_logic_vector(unsigned(q_softstart_counter) + 1);
			clock_enable <= '0';
		elsif q_softstart_counter = std_logic_vector(to_unsigned(delay_const,10)) then
			d_softstart_counter <= (others => '0');
			if q_softstart_stage = '0' then
				--d_softstart_counter <= (others => '0');
				clock_enable <= '1';
			end if;	
		end if; 
	end process;

	process(clock_enable,q_softstart_out,q_softstart_stage)
	begin	
		d_softstart_out <= q_softstart_out;
		d_softstart_stage <= q_softstart_stage;
		if clock_enable = '1' then
			d_softstart_out <= std_logic_vector(unsigned(q_softstart_out) - 1);
		end if;
		if q_softstart_out = std_logic_vector(to_unsigned(output_final_value,10)) then
			d_softstart_stage <= '1';															-------change to the non-counting state
		end if;	
	end process;
	
	process(duty_cycle,q_softstart_out)
	begin
	  if (duty_cycle < q_softstart_out) then
	    softstart_out <= q_softstart_out;
	  else
       softstart_out <= duty_cycle;
     end if;
   end process;	
	
	
end Behavioral;

