----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:08:26 04/03/2016 
-- Design Name: 
-- Module Name:    spi_master_out - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;



entity spi_master_out is
	 generic (	transaction_width : natural := 2; data_width : natural := 12; 
					address_width : natural := 4; data_width_2 : natural := 4);
    Port ( --transaction_size : in  STD_LOGIC_VECTOR (transaction_width-1 downto 0);
          -- polarity : in  STD_LOGIC;
			  rst 							: in std_logic;
			  clk50 							: in std_logic;	---50MHz input clock
			  clk5 							: in std_logic;		---5MHz input clock
           data_in 						: in  STD_LOGIC_VECTOR (data_width_2-1 downto 0);
			  data_out 						: out std_logic;
			  miso 							: in std_logic;
			  read_data 					: out std_logic_vector (data_width-1 downto 0);
			  chip_select 					: out std_logic;
			  timeout,clk_adc 			: out std_logic;
			  new_data_available 		: in std_logic;
           busy 							: out  STD_LOGIC);
end spi_master_out;

architecture Behavioral of spi_master_out is
-----counter which counts number of bits sent
constant bit_counter_width : natural := 4;
-----
constant max_bit_count : natural := 12;
constant max_bit_count_2 : natural := 4;
constant timeout_const_width : natural := 16;
constant timeout_const : natural := 32767;
-----flip flop, which stores data from the control block
signal q_input_flip_flop : std_logic_vector(data_width_2-1 downto 0);
signal d_input_flip_flop : std_logic_vector(data_width_2-1 downto 0);
-----flip-flop, which stores the incoming address
--signal q_address_in, d_address_in : std_logic_vector(address_width-1 downto 0);
-----flip-flop, which stores the number of bits already sent
signal q_bit_count, d_bit_count : std_logic_vector(bit_counter_width-1 downto 0);
-----signal, which tells that new data is about to be sent
--signal new_data_available : std_logic;
-----signal, which tells to the control block that it cannot send new data
signal spi_busy : std_logic;
-----flip-flop used as output shift register
signal q_spi_output_buffer,d_spi_output_buffer : std_logic_vector(data_width_2-1 downto 0);
-----timeout counter when slave does not respond
signal q_latency_timer, d_latency_timer : std_logic_vector(timeout_const_width-1 downto 0);
signal q_spi_read_data_buffer : std_logic_vector(data_width-1 downto 0); signal d_spi_input_buffer,q_spi_input_buffer : std_logic_vector(data_width-1 downto 0); signal d_spi_read_data_buffer : std_logic_vector(data_width-1 downto 0); 
-----SPI transceiver FSM states
type state is (s_idle, s_start, s_sending, s_stop, s_load_2, s_read, s_timeout);
signal present_state,next_state : state; 
signal EN : std_logic;
signal cnt : std_logic_vector(4 downto 0);
begin

------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------
--synchronous part

sync_part_50 : process(clk50,rst)
	begin
		if rst = '1' then
			q_input_flip_flop <= (others => '0');
			q_spi_read_data_buffer <= (others => '0');
	--		q_address_in <= (others => '0');
		elsif rising_edge(clk50) then
			q_input_flip_flop <= d_input_flip_flop;
			q_spi_read_data_buffer <= d_spi_read_data_buffer;
	--		q_address_in <= d_address_in;
		end if;			
end process;



clock_divider: process(clk50, rst)
  begin
    if rst = '1' then
	   cnt <= (others => '0');
		EN <= '0';
	 elsif falling_edge(clk50) then
      if cnt < std_logic_vector(to_unsigned(24,5)) then
		  EN <= '0';
		  cnt <= std_logic_vector(unsigned(cnt) + 1);	 
		else 
        cnt <= (others => '0');
		  EN <= '1';
      end if;
		if cnt > std_logic_vector(to_unsigned(11,5)) and cnt < std_logic_vector(to_unsigned(24,5)) then
		  clk_adc <= '1';
		else
        clk_adc <= '0';
      end if;		  
    end if;
  end process;	 

sync_part_5 : process(clk50,rst)
	begin
		if rst = '1' then
			q_spi_output_buffer <= (others => '0');
			present_state <= s_idle;
			q_latency_timer <= (others => '0');
			q_bit_count <= (others => '0'); 
		elsif falling_edge(clk50) and EN = '1' then
			q_spi_output_buffer <= d_spi_output_buffer;
			present_state <= next_state;
			q_latency_timer <= d_latency_timer;
			q_bit_count <= d_bit_count; 
		end if;
end process;	

sync_part_5_read : process(clk50,rst)
	begin
		if rst = '1' then
			q_spi_input_buffer <= (others => '0');
		elsif falling_edge(clk50) and EN = '1' then
			q_spi_input_buffer <= d_spi_input_buffer;
		end if;	
end process;
------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------	
------------------------------------------------------------------------------------------------------------
--combinational part

combinational_part_5 : process(q_spi_output_buffer,new_data_available,q_latency_timer,q_bit_count,q_input_flip_flop,present_state,q_spi_input_buffer,q_spi_read_data_buffer,miso)
begin
	d_spi_output_buffer <= q_spi_output_buffer;
	next_state <= present_state;
	timeout <= '0';
	spi_busy <= '0';
	chip_select <= '1';
	d_latency_timer <= q_latency_timer;
	d_bit_count <= q_bit_count;
	d_spi_input_buffer <= q_spi_input_buffer;
	d_spi_read_data_buffer <= q_spi_read_data_buffer;
	case present_state is
		when s_idle =>
		---when controller sends new data, set busy flag
			if new_data_available = '1' then
				spi_busy <= '1';
				d_spi_output_buffer <= (others => '1');
				next_state <= s_start;
			else 
            spi_busy <= '0';			
			end if;
			
		---pull CS low to activate slave, hold CS low	
		when s_start =>
			chip_select <= '0';
			spi_busy <= '1';
		--	next_state <= s_load;
		--when s_load =>
			--chip_select <= '0';
			--spi_busy <= '1';
			d_spi_output_buffer <= q_input_flip_flop;
			next_state <= s_sending;
		---shift bits to the output, hold CS low	
		when s_sending =>	
			chip_select <= '0'; 
			spi_busy <= '1';
			d_bit_count <= std_logic_vector(unsigned(q_bit_count) + 1); 
			d_spi_output_buffer <= '0'& q_spi_output_buffer(max_bit_count_2-1 downto 1);
			if q_bit_count = std_logic_vector(to_unsigned(max_bit_count-1,bit_counter_width)) then 
				next_state <= s_stop;
			end if;	
		when s_stop =>
			spi_busy <= '1';
			chip_select <= '0';
			d_bit_count <= (others => '0');
			d_latency_timer <= std_logic_vector(unsigned(q_latency_timer) + 1);
			if d_latency_timer > std_logic_vector(to_unsigned(timeout_const,timeout_const_width)) then
			--	d_latency_timer <= std_logic_vector(unsigned(q_latency_timer) + 1);
			--elsif d_latency_timer > std_logic_vector(to_unsigned(timeout_const,timeout_const_width)) then
				next_state <= s_timeout;
			--	d_latency_timer <= (others => '0');
			end if;	
			if miso = '0' then
				next_state <= s_read;
			end if;
		when s_read =>
			spi_busy <= '1';
			chip_select <= '0';
			d_bit_count <= std_logic_vector(unsigned(q_bit_count) + 1);
			d_spi_input_buffer <= q_spi_input_buffer(max_bit_count-1-1 downto 0) & miso;
			if q_bit_count = std_logic_vector(to_unsigned(max_bit_count-1,bit_counter_width)) then 
				next_state <= s_load_2;
				d_bit_count <= (others => '0');
			end if;			
		when s_load_2 =>
			spi_busy <= '1';
			chip_select <= '1';
			d_spi_read_data_buffer <= q_spi_input_buffer;
			next_state <= s_idle;
		when s_timeout =>
			timeout <= '1';
			d_latency_timer <= (others => '0');
			next_state <= s_idle;
		when others => 
		   next_state <= s_idle;
	end case;
end process;	

--d_input_flip_flop <= data_in;
busy <= spi_busy;
data_out <= q_spi_output_buffer(0);
--d_spi_input_buffer <= miso;
read_data <= q_spi_read_data_buffer;
reverse_bits: for i in 0 to max_bit_count_2-1 generate
						d_input_flip_flop(max_bit_count_2-1-i) <= data_in(i);
				  end generate;
end Behavioral;

