----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    18:00:33 03/20/2016 
-- Design Name: 
-- Module Name:    top_block - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_unsigned.all;
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity top_block is
    Port ( --freq_control,freq_control_2 : in  STD_LOGIC_VECTOR (31 downto 0);
           clk : in  STD_LOGIC;
			  --SCK : out std_logic;
           rst : in  STD_LOGIC;
			  SDO : out std_logic;
			  SDI : in std_logic;

			  nCS : out std_logic;
			  SCK : out std_logic;
			  boost_pwm_out : out std_logic_vector(3 downto 0);--sine_amplitude : std_logic_vector(9 downto 0);
           pwm_out_1,pwm_out_2,pwm_out_3,pwm_out_4 : out  STD_LOGIC);
end top_block;

architecture Behavioral of top_block is
    signal adc_data : std_logic_vector(11 downto 0);
	 signal new_data_available : std_logic;
	 signal spi_busy : std_logic;
	 signal vin,iin : std_logic_vector(7 downto 0);
   signal ch_n : std_logic_vector(3 downto 0);
	signal phase,phase_2,softstart_out,duty : std_logic_vector(9 downto 0);
	signal sine_signal_unlimited : std_logic_vector(9 downto 0);
	signal sine_sample : std_logic_vector(9 downto 0);
	COMPONENT sine_phase_acc
	PORT(
		freq_control,freq_control_2 : IN std_logic_vector(31 downto 0);
		clk : IN std_logic;
		rst : IN std_logic;          
		phase_out,phase_out_2 : OUT std_logic_vector(9 downto 0)
		);
	END COMPONENT;
	
	COMPONENT MPPT_block
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;
		new_adc_data : IN std_logic;
		vin : IN std_logic_vector(7 downto 0);
		iin : IN std_logic_vector(7 downto 0);          
		d_out : OUT std_logic_vector(9 downto 0)
		);
	END COMPONENT;
	
	COMPONENT ADC_channel_selector
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;
		spi_busy : IN std_logic;
		data_in : IN std_logic_vector(11 downto 0);          
		new_data_available : OUT std_logic;
		channel_number : OUT std_logic_vector(3 downto 0);
		vin : OUT std_logic_vector(7 downto 0);
		iin : OUT std_logic_vector(7 downto 0);
		output_sine : OUT std_logic_vector(7 downto 0);
		mains_sine : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;

	
	COMPONENT sine_rom_memory
	PORT(
		phase_in : IN std_logic_vector(9 downto 0);
		clk : IN std_logic;
		rst : IN std_logic;          
		sine_out : OUT std_logic_vector(9 downto 0)
		);
	END COMPONENT;
	
		COMPONENT spi_master_out
	PORT(
		rst : IN std_logic;
		clk50 : IN std_logic;
		clk5 : IN std_logic;
		data_in : IN std_logic_vector(3 downto 0);
		miso : IN std_logic;
		new_data_available : IN std_logic;          
		data_out : OUT std_logic;
		read_data : OUT std_logic_vector(11 downto 0);
		chip_select : OUT std_logic;
		timeout : OUT std_logic;
		clk_adc : OUT std_logic;
		busy : OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT softstart_circuit
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;   
      duty_cycle : IN std_logic_vector(9 downto 0);		
		softstart_out : OUT std_logic_vector(9 downto 0)
		);
	END COMPONENT;
	
		COMPONENT sine_sample_limiter
	PORT(
		clk,rst : IN std_logic;
		sine_in,sine_amplitude : IN std_logic_vector(9 downto 0);		
		sine_out : OUT std_logic_vector(9 downto 0)
		);
	END COMPONENT;
	
		COMPONENT sine_pwm_counter
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;
		phase_in : std_logic_vector(9 downto 0);
		sine_sample_in : IN std_logic_vector(9 downto 0);          
		pwm_out_1 : OUT std_logic;
		pwm_out_2 : OUT std_logic;
		pwm_out_3 : out std_logic;
		pwm_out_4 : out std_logic
		);
	END COMPONENT;
	
		COMPONENT boost_pwm_counter
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;
		duty_cyc_in : IN std_logic_vector(9 downto 0);
		shutdown : IN std_logic;          
		gate_pulse_out : OUT std_logic_vector(3 downto 0)
		);
	END COMPONENT;

	

begin

	Inst_sine_phase_acc: sine_phase_acc PORT MAP(
		freq_control => std_logic_vector(to_unsigned(8590,32)),--freq_control	
		freq_control_2 => (std_logic_vector(to_unsigned(1736000,32))),
		clk => clk,
		rst => rst,
		phase_out_2 => phase_2,
		phase_out => phase 
	);
	
		Inst_sine_rom_memory: sine_rom_memory PORT MAP(
		phase_in => phase,
		clk => clk,
		rst => rst,
		sine_out => sine_signal_unlimited 
	);
	
		Inst_sine_sample_limiter: sine_sample_limiter PORT MAP(
		sine_in => sine_signal_unlimited,
		clk => clk,
		rst => rst,
		sine_amplitude => std_logic_vector(to_unsigned(800,10)),
		sine_out => sine_sample
	);
	
		Inst_sine_pwm_counter: sine_pwm_counter PORT MAP(
		clk => clk,
		rst => rst,
		phase_in => phase_2,
		sine_sample_in => sine_sample,
		pwm_out_1 => pwm_out_1,
		pwm_out_2 => pwm_out_2,
		pwm_out_3 => pwm_out_3,
		pwm_out_4 => pwm_out_4
	);
	
		Inst_softstart_circuit: softstart_circuit PORT MAP(
		clk => clk,
		rst => rst,
		duty_cycle => duty,
		softstart_out => softstart_out
	);
  

	Inst_boost_pwm_counter: boost_pwm_counter PORT MAP(
		clk => clk,
		rst => rst,
		duty_cyc_in => softstart_out,
		shutdown => '0',
		gate_pulse_out => boost_pwm_out
	);
	
		Inst_spi_master_out: spi_master_out PORT MAP(
		rst => rst,
		clk50 => clk,
		clk5 => '0',
		data_in => ch_n,
		data_out => SDO,
		miso => SDI,
		read_data => adc_data,
		chip_select => nCS,
		timeout => open,
		clk_adc => SCK,
		new_data_available => new_data_available,
		busy => spi_busy
	);
	
	Inst_MPPT_block: MPPT_block PORT MAP(
		clk => clk,
		rst => rst,
		new_adc_data => '1',
		vin => vin,
		iin => vin,
		d_out => duty
	);

	Inst_ADC_channel_selector: ADC_channel_selector PORT MAP(
		clk => clk,
		rst => rst,
		spi_busy => spi_busy,
		data_in => adc_data,
		new_data_available => new_data_available,
		channel_number => ch_n,
		vin => vin,
		iin => iin,
		output_sine => open,
		mains_sine => open
	);

end Behavioral;

